unit LocalBrowser;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, OleCtrls, Isapi, ComCtrls, ShellAPI, ExtCtrls, StdCtrls, Grids,
  WABD_Objects, Compat,
  Test_Forms;     // Remove this

const
   WM_DISPLAYFORM = WM_USER + 1;
   WM_SUBMITFIRST = WM_USER + 2;
   WM_GETSTATUS   = WM_USER + 3;

type
  TLocalBrowserForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    N1: TMenuItem;
    ViewSource1: TMenuItem;
    StatusBar1: TStatusBar;
    Reconnect1: TMenuItem;
    View1: TMenuItem;
    StatusBar2: TMenuItem;
    N3: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    Options1: TMenuItem;
    RunMultithreaded1: TMenuItem;
    N5: TMenuItem;
    WABDReadme1: TMenuItem;
    N2: TMenuItem;
    PreviewHTML1: TMenuItem;
    ViewBox: TScrollBox;
    CurrentStatus1: TMenuItem;
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ViewSource1Click(Sender: TObject);
    procedure Reconnect1Click(Sender: TObject);
    procedure View1Click(Sender: TObject);
    procedure StatusBar2Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure WABDReadme1Click(Sender: TObject);
    procedure RunMultithreaded1Click(Sender: TObject);
    procedure PreviewHTML1Click(Sender: TObject);
    procedure ButtonClick(Sender: TObject);
    procedure ImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure StringGridSelectCell(Sender: TObject; Col, Row: longint;
      var CanSelect: Boolean);
    procedure StringGridDblClick(Sender: TObject);
    procedure CurrentStatus1Click(Sender: TObject);
  private
    { Private declarations }
    FPaintBack : boolean;
    procedure  SetStat(i:integer; s: string);
    procedure  AppHint(Sender: TObject);
    procedure  RunHttpProcThread(var ECB: TEXTENSION_CONTROL_BLOCK);
    procedure  DisplayFormMsg(var Msg: TMessage); message WM_DISPLAYFORM;
    procedure  SubmitFirst(var Msg: TMessage); message WM_SUBMITFIRST;
    procedure  GetStatusProc(var Msg: TMessage); message WM_GETSTATUS;
    procedure  SetPaintBack(b: boolean);
    procedure  JumpLabelClick(Sender: TObject);
  public
    { Public declarations }
    BackPic    : TImage;
    LastRow    : integer;
    GetStatus  : boolean;
    property   PaintBack: boolean read FPaintBack write SetPaintBack;
    procedure  DisplayForm;
    procedure  SubmitQuery(QueryStr: string);
    property   Stat[i: integer]: string write SetStat;
    function   EncodeFormURL: string;
    procedure  ClearViewPanel;
    procedure  SetSpecialProperties(c: TControl);
    procedure  SetAllProperties(c: TWinControl);
  end;

var
  LocalBrowserForm: TLocalBrowserForm;

implementation

{$R *.DFM}

{$IFDEF VER100}
   uses WABD_ISAPI, TraceUnit, ViewSource, WABD_About, HTML_Input, JPEG;
{$ELSE}
   uses WABD_ISAPI, TraceUnit, ViewSource, WABD_About, HTML_Input;
{$ENDIF}


type
   TServerSimThread = class(TThread)
   protected
      ECB      : TEXTENSION_CONTROL_BLOCK;
      procedure Execute; override;
   end;


var
   TmpName : string;

// ************************************************************************

procedure TLocalBrowserForm.SubmitFirst(var Msg: TMessage);
begin
   SubmitQuery('');
end;

procedure TLocalBrowserForm.GetStatusProc(var Msg: TMessage);
begin
   ViewSource1Click(Self);
end;

procedure TLocalBrowserForm.SetPaintBack(b: boolean);
begin
   FPaintBack := b;
end;

procedure TLocalBrowserForm.SetStat(i:integer; s: string);
begin
   StatusBar1.Panels[i].Text := s;
   StatusBar1.Update;
end;

procedure TLocalBrowserForm.Exit1Click(Sender: TObject);
begin
   Close;
end;


function LocalGetServerVariable(hConn: HCONN; VariableName: PChar;
   Buffer: Pointer; var Size: DWORD): BOOL stdcall;
   procedure SafeCopy(s: string);
   begin
      StrLCopy(PChar(Buffer), PChar(s), Size-1);
      Size := Length(s);
   end;
begin
   // TRACE('Checking for Variable: %s', [VariableName]);
   Result := False;
   if VariableName = 'SCRIPT_NAME' then begin
      SafeCopy('/scripts/LocalBrowser.dll');
      Result := True;
   end;
   if VariableName = 'REMOTE_ADDR' then begin
      SafeCopy('Local Address');
      Result := True;
   end;
   if VariableName = 'REMOTE_HOST' then begin
      SafeCopy('Local Host');
      Result := True;
   end;
   if VariableName = 'REMOTE_USER' then begin
      SafeCopy('Local User');
      Result := True;
   end;
end;

function LocalWriteClient(ConnID: HCONN; Buffer: Pointer; var Bytes: DWORD;
   dwReserved: DWORD ): BOOL stdcall;
const
   Srch = '<HTML>';
var
   s        : string;
   p        : integer;
   fs       : TFileStream;
   bf       : TLocalBrowserForm;
begin
   TmpName := 'LocalTemp.htm';
   s := PChar(Buffer);
   p := Pos(Srch, s);
   s := Copy(s, p, Length(s));

   fs := TFileStream.Create(TmpName, fmCreate or fmOpenWrite or fmShareDenyWrite);
   fs.Write(s[1], Length(s));
   fs.Free;

   bf := TLocalBrowserForm(ConnID);
   if bf.GetStatus then begin
      bf.GetStatus := False;
      PostMessage(bf.Handle, WM_GETSTATUS, 0, 0);
   end else begin
      PostMessage(bf.Handle, WM_DISPLAYFORM, Length(s), 0);
   end;

   Result := True;
end;

procedure TLocalBrowserForm.SubmitQuery(QueryStr: string);
var
   LocalECB : TEXTENSION_CONTROL_BLOCK;
begin
   with LocalECB do begin
      cbSize               := sizeof(LocalECB);
      dwVersion            := $DEADF00D;                 // MAGIC number
      ConnID               := integer(self);
      lpszMethod           := '';
      lpszPathInfo         := '';
      lpszPathTranslated   := '';
      lpszQueryString      := '';
      lpbData              := PChar(QueryStr);
      cbTotalBytes         := Length(QueryStr);
      cbAvailable          := Length(QueryStr);

      GetServerVariable     := LocalGetServerVariable;
      WriteClient           := LocalWriteClient;
      ReadClient            := nil;
      ServerSupportFunction := nil;
   end;

   // TRACE('Sending Query to ISAPI Extension DLL: %s', [QueryStr]);
   Screen.Cursor := crHourGlass;
   Stat[0] := 'Submitting Query';
   Stat[1] := QueryStr;
   if RunMultithreaded1.Checked then begin
      RunHttpProcThread(LocalECB);
   end else begin
      HttpExtensionProc(LocalECB);
   end;
   Stat[0] := 'Formatting HTML';
   Screen.Cursor := crDefault;
end;

procedure TLocalBrowserForm.FormCreate(Sender: TObject);
begin
   GetStatus := False;

   BackPic := TImage.Create(Self);
   BackPic.AutoSize := True;
   BackPic.Name := BACKGND_PIC_NAME;

   FPaintBack := False;
   LocalBrowserLogoff := False;
   Application.OnHint := AppHint;
   Show;
   PostMessage(Handle, WM_SUBMITFIRST, 0, 0);
end;

procedure TLocalBrowserForm.ViewSource1Click(Sender: TObject);
var
   vf : TViewSourceForm;
begin
   vf := TViewSourceForm.Create(self);
   vf.Memo1.Lines.LoadFromFile(TmpName);
   vf.ShowModal;
end;

procedure TLocalBrowserForm.Reconnect1Click(Sender: TObject);
begin
   SubmitQuery('');
end;

procedure TLocalBrowserForm.View1Click(Sender: TObject);
begin
   StatusBar2.Checked := StatusBar1.Visible;
end;

procedure TLocalBrowserForm.StatusBar2Click(Sender: TObject);
begin
   StatusBar2.Checked := not StatusBar2.Checked;
   StatusBar1.Visible := StatusBar2.Checked;
end;

procedure TLocalBrowserForm.AppHint(Sender: TObject);
begin
   Stat[1] := Application.Hint;
end;

procedure TLocalBrowserForm.About1Click(Sender: TObject);
var
   af : TForm;
begin
   af := TWABDAboutForm.Create(self);
   af.ShowModal;
end;

procedure TLocalBrowserForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
   rc : integer;
begin
   if not LocalBrowserLogoff then begin
      rc := MessageDlg('Are you sure you want to exit without properly logging off?',
         mtWarning, mbYesNoCancel, 0);

      CanClose := rc = mrYes;
   end;
end;

procedure TLocalBrowserForm.WABDReadme1Click(Sender: TObject);
var
   s : string;
   rc : integer;
begin
   s := 'C:\WABD\Web Application Builder for Delphi.doc';
   rc := ShellExecute(handle, 'open', PChar(s), nil, nil, SW_SHOWNORMAL);
   if (rc <= 32) then ShowMessage('Unable to execute: ' + s + #13#10 + 'Browser not installed?');
end;

procedure TLocalBrowserForm.RunHttpProcThread(var ECB: TEXTENSION_CONTROL_BLOCK);
var
   st  : TServerSimThread;
begin
   st                 := TServerSimThread.Create(True);
   st.ECB             := ECB;
   st.FreeOnTerminate := False;
   st.Resume;

   st.WaitFor;
   st.Free;
end;

procedure TServerSimThread.Execute;
begin
   HttpExtensionProc(ECB);
end;

procedure TLocalBrowserForm.RunMultithreaded1Click(Sender: TObject);
begin
   RunMultiThreaded1.Checked := not RunMultiThreaded1.Checked;
end;

procedure TLocalBrowserForm.PreviewHTML1Click(Sender: TObject);
var
   rc : integer;
begin
   rc := ShellExecute(handle, 'open', PChar(TmpName), nil, nil, SW_SHOWNORMAL);
   if (rc <= 32) then ShowMessage('Unable to execute: ' + TmpName + #13#10 + 'Browser not installed?');
end;

procedure TLocalBrowserForm.DisplayFormMsg(var Msg: TMessage);
begin
   Stat[1] := Format('  HTML Length (bytes) = %1.0n', [Msg.wParam+0.0]);
   if LocalBrowserLogoff then begin
      Close;
   end else begin
      DisplayForm;
   end;
end;

procedure TLocalBrowserForm.ClearViewPanel;
var
   i : integer;
begin
   for i := ViewBox.ControlCount-1 downto 0 do begin
      ViewBox.Controls[i].Free;
   end;
end;


procedure TLocalBrowserForm.DisplayForm;
var
   f : TWABD_Form;
   c : TControl;
begin
   ClearViewPanel;

   if LocalBrowserException<>'' then
      raise Exception.Create(LocalBrowserException);

   f := GetLocalBrowserForm;

   if f.BgrdImage<>nil then begin
      try
         PaintBack := True;
         BackPic.Picture.LoadFromFile(f.BgrdImage.LocalPath + f.BgrdImage.ImagePath);
      except
         PaintBack := False;
      end;
   end else begin
      PaintBack := False;
   end;

   c := f.Object_To_Control(ViewBox);
   if c.Width < ViewBox.ClientWidth then
      c.Width := ViewBox.ClientWidth;
   if c.Height < ViewBox.ClientHeight then
      c.Height := ViewBox.ClientHeight;

   SetAllProperties(ViewBox);
end;

procedure TLocalBrowserForm.SetSpecialProperties(c: TControl);
begin
   if (c is TButton) then begin
      (c as TButton).OnClick := ButtonClick;
   end;

   if (c is TPaintPanel) then begin
      (c as TPaintPanel).Pic := BackPic;
      (c as TPaintPanel).DrawPic := PaintBack;
   end;

   if (c is TImage) then begin
      (c as TImage).OnMouseDown := ImageMouseDown;
   end;

   if (c is TStringGridEx) then begin
      (c as TStringGridEx).OnSelectCell := StringGridSelectCell;
      (c as TStringGridEx).OnDblClick   := StringGridDblClick;
   end;

   if (c is TJumpLabel) then begin
      if (c as TJumpLabel).CanClick then
         (c as TJumpLabel).OnClick := JumpLabelClick; 
   end;
end;

procedure TLocalBrowserForm.SetAllProperties(c: TWinControl);
var
   i     : integer;
   Child : TControl;
begin
   SetSpecialProperties(c);
   for i := 0 to c.ControlCount-1 do begin
      Child := c.Controls[i];
      if (Child is TWinControl) then begin
         SetAllProperties(Child as TWinControl);
      end else begin
         SetSpecialProperties(Child);
      end;
   end;
end;

procedure TLocalBrowserForm.ButtonClick(Sender: TObject);
var
   b : TButton;
   s : string;
begin
   b := Sender as TButton;
   s := Format('%s=%s', [WABD_BUTTON_STR, EncodeEscapes(b.Caption)]);
   SubmitQuery(EncodeFormURL+s);
end;

procedure TLocalBrowserForm.StringGridDblClick(Sender: TObject);
var
   sg : TStringGridEx;
   s  : string;
begin
   sg := Sender as TStringGridEx;
   if not sg.CanClick then exit;
   s := Format('%s=%s%d', [sg.Name, EncodeEscapes(sg.ClickText+' '), LastRow]);
   SubmitQuery(EncodeFormURL+s);
end;

procedure TLocalBrowserForm.ImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   n, s : string;
begin
   n := (Sender as TImage).Name;
   s := Format('%s.X=%d&%s.Y=%d', [n, X, n, Y]);
   SubmitQuery(EncodeFormURL + s);
end;

procedure EncodeControl(c: TControl; var t: string);
var
   lb    : TListBox;
   cb    : TComboBox;
   s     : string;
begin
   if (c is TEdit) then begin
      t := t + c.Name + '=' + EncodeEscapes((c as TEdit).Text) + '&';
   end;

   if (c is TListBox) then begin
      lb := c as TListBox;
      if lb.ItemIndex<>-1 then begin
         s := lb.Items[lb.ItemIndex];
         t := t + c.Name + '=' + EncodeEscapes(s) + '&';
      end;
   end;

   if (c is TCheckBox) then begin
      if (c as TCheckBox).Checked then begin
         t := t + c.Name + '=CHECKED&';
      end;
   end;

   if (c is TMemo) then begin
      t := t + c.Name + '=' + EncodeEscapes((c as TMemo).Lines.Text) + '&';
   end;

   if (c is TRadioButton) then begin
      if (c as TRadioButton).Checked then begin
         t := t + WABD_RADIO_STR + '=' + (c as TRadioButton).Name + '&';
      end;
   end;

   if (c is TComboBox) then begin
      cb := c as TComboBox;
      if cb.ItemIndex<>-1 then begin
         s := cb.Items[cb.ItemIndex];
         t := t + c.Name + '=' + EncodeEscapes(s) + '&';
      end;
   end;
end;

procedure EncodeAllControls(p: TWinControl; var t: string);
var
   i : integer;
   c : TControl;
begin
   EncodeControl(p, t);
   for i := 0 to p.ControlCount-1 do begin
      c := p.Controls[i];
      if c is TWinControl then
         EncodeAllControls(c as TWinControl, t)
      else
         EncodeControl(c, t);
   end;
end;

function TLocalBrowserForm.EncodeFormURL: string;
var
   t     : string;
begin
   t := Format('%s=%d', [WABD_SES_ID_STR, GetLocalBrowserForm.Session.SessionID]) + '&';

   EncodeAllControls(ViewBox, t);

   Result := t;
end;

procedure TLocalBrowserForm.FormDestroy(Sender: TObject);
begin
   BackPic.Free;
end;

procedure TLocalBrowserForm.StringGridSelectCell(Sender: TObject; Col,
  Row: longint; var CanSelect: Boolean);
begin
   LastRow := Row;
end;

procedure TLocalBrowserForm.CurrentStatus1Click(Sender: TObject);
begin
   GetStatus := True;
   SubmitQuery('WABDSTATUS=1');
end;

procedure TLocalBrowserForm.JumpLabelClick(Sender: TObject);
var
   jl : TJumpLabel;
   s  : string;
   rc : integer;
begin
   jl := Sender as TJumpLabel;

   s := jl.JumpDest;

   if jl.JumpOut then begin
      rc := ShellExecute(handle, 'open', PChar(s), nil, nil, SW_SHOWNORMAL);
      if (rc <= 32) then ShowMessage('Unable to execute: ' + s + #13#10 + 'Browser not installed?');
   end else begin
      s := Copy(s, Pos('?', s)+1, Length(s));
      SubmitQuery(s);
   end;
end;

end.
